@@ -17,6 +17,9 @@ from avr import YamahaRXVClient
 from time import sleep
 import socket
 from telnetlib import Telnet
+import operator
+import random
+from decimal import *
 
 class TrackChangedEvent(QEvent):
 	def __init__(self, ix):
@@ -54,7 +57,7 @@ class CurrentSongThread ( QThread ):
 			self.event.setCurrentSongIndex(res.MResult)
 			qApp.postEvent(self.gui,self.event)
 		except:
-			print "Failed to get Current Song"	
+			print "Exception: Failed to get Current Song"	
 			traceback.print_exc(None, None)
 
 
@@ -62,12 +65,25 @@ class CurrentSongThread ( QThread ):
 class XPotMainWindow (QMainWindow):
 	"""XPotMainWindow inherits QMainWindow"""
 	def __init__ (self, parent = None):
-		self.version = "0.2"
+		self.version = "0.4"
 		self.name = "XBMC Potato"
 		self.developer = "Yiannis Petevis"
 		self.email = "y.petevis@gmail.com"
 		QMainWindow.__init__(self, parent)
 		self.setWindowTitle(self.name)
+		self.directoryClicked = False
+		self.randomize = False
+		self.randomizeCap = 50
+		
+		self.progressCancelButton = QPushButton("Cancel")
+		self.progDialogAdd = QProgressDialog(self)
+		self.progDialogAdd.setCancelButton(self.progressCancelButton)
+		self.progDialogAdd.setAutoReset(True)
+		self.progDialogAdd.setAutoClose(True)
+		self.progDialogAdd.setMinimum(0)
+		self.progDialogAdd.setMaximum(100)
+		self.progDialogAdd.setWindowModality(QtCore.Qt.WindowModal)
+		self.progDialogAddCancelled = False
 		
 		self.progDialog = QProgressDialog(self)
 		self.progDialog.setAutoReset(True)
@@ -92,9 +108,7 @@ class XPotMainWindow (QMainWindow):
 		self.xbmc = XBMCLib(self.config.server, self.config.port, self.config.user, self.config.password)			
 		if self.config.server == "":
 			self.progDialog.setValue(100)	
-			qd = QMessageBox(self)
-			qd.setText("XBMC Potato has not been configured yet.  Click OK to configure now and restart the application.")
-			qd.exec_()
+			self.showMessage("XBMC Potato has not been configured yet.  Click OK to configure now and restart the application.")
 			self.showToolsDialog()
 			return
 
@@ -107,8 +121,8 @@ class XPotMainWindow (QMainWindow):
 				sleep(5)
 			else:
 				self.progDialog.setWindowTitle("Waiting for XBMC to respond...")		
-				self.progDialog.setValue(20)					
-			#self.isXBMCPoweredOn()
+				self.progDialog.setValue(20)				
+				self.config.powerOnXBMCTimeout = 2
 			end = time.time()
 			elapsed = end - start
 			if elapsed >= int(self.config.powerOnXBMCTimeout):
@@ -138,9 +152,7 @@ class XPotMainWindow (QMainWindow):
 			self.progDialog.setValue(100)		
 		else:
 			self.progDialog.setValue(100)
-			qd = QMessageBox(self)
-			qd.setText("Connection to XBMC failed")
-			qd.exec_
+			self.showMessage("Connection to XBMC failed")
 
 	def isXBMCPoweredOn(self):
 		s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
@@ -192,13 +204,10 @@ class XPotMainWindow (QMainWindow):
 					while x <  self.ui.treeWidgetPlaylist.topLevelItemCount():
 						if x <> ix:
 							color = QColor("gray")
-							#self.ui.treeWidgetPlaylist.setItemSelected(self.ui.treeWidgetPlaylist.topLevelItem(x), False)
 						else:
 							color = QColor()
 							color.setNamedColor("#FF6600")
 							self.ui.treeWidgetPlaylist.scrollToItem(self.ui.treeWidgetPlaylist.topLevelItem(x))
-							#self.ui.treeWidgetPlaylist.setItemSelected(self.ui.treeWidgetPlaylist.topLevelItem(x), True)
-							#color.setBlue(255)
 						brush = QBrush()
 						brush.setColor(color)
 						self.ui.treeWidgetPlaylist.topLevelItem(x).setForeground(0,brush)
@@ -213,13 +222,10 @@ class XPotMainWindow (QMainWindow):
 			rootButton = QPushButton()
 			rootButton.setIcon(QIcon(":/img/user-home.png"))
 			rootButton.setCheckable(True)
-			#rootButton.setSizePolicy(QSizePolicy(QSizePolicy.Expanding))
 			rootButton.setIconSize(QSize(32,32))
 			self.ui.navLayout.addWidget(rootButton,0)
 			rootButton.clicked.connect(self.mapper.map)
 			self.mapper.setMapping(rootButton, "|GOTO_SHARES_ROOT")
-			#spacer = QSpacerItem(0,0,QSizePolicy.Expanding,QSizePolicy.Minimum)
-			#self.ui.navLayout.addSpacerItem(spacer)
 	
 	def getCurrentPlaylistItems(self):
 		itemsList = []
@@ -236,33 +242,34 @@ class XPotMainWindow (QMainWindow):
 			self.ui.treeWidgetPlaylist.insertTopLevelItems(ix, itemsList)
 			self.ui.treeWidgetPlaylist.setColumnCount(1)
 		else:
-			#self.ui.statusBar.showMessage(items.MText)
 			print items.MText
+		self.ui.labelPlaylist.setText("Tracks: %s" % str(self.ui.treeWidgetPlaylist.topLevelItemCount()))
 		
 	def signalWiring(self):
 		self.ui.pushButtonAdd.clicked.connect(self.addSelectedSongs)
 		self.ui.pushButtonAddAll.clicked.connect(self.addAllSongs)
+		self.ui.pushButtonRandomize.clicked.connect(self.addRandomSongs)
 		self.ui.pushButtonPlay.clicked.connect(self.playSongs)
 		self.ui.pushButtonStop.clicked.connect(self.stopPlaying)
 		self.ui.pushButtonClear.clicked.connect(self.clearPlaylist)
 		self.ui.pushButtonSkipNext.clicked.connect(self.skipNext)
 		self.ui.pushButtonSkipPrevious.clicked.connect(self.skipPrevious)
 		self.ui.pushButtonShuffle.toggled.connect(self.shuffle)
+		self.ui.treeWidget.itemChanged.connect(self.clickDirectory)
 		self.ui.treeWidget.itemClicked.connect(self.click)
-		#self.ui.treeWidget.itemActivated.connect(self.click)
 		self.ui.pushButtonFullScreenToggle.clicked.connect(self.toggleFullScreen)
 		self.ui.pushButtonTools.clicked.connect(self.showToolsDialog)
 		self.ui.pushButtonVolUp.pressed.connect(self.volumeUp)
 		self.ui.pushButtonVolUp.released.connect(self.volumeUpStop)
 		self.ui.pushButtonVolDown.pressed.connect(self.volumeDown)
 		self.ui.pushButtonVolDown.released.connect(self.volumeDownStop)
+		self.progDialogAdd.canceled.connect(self.cancelProgressDialog)
 	
 
 	def __del__ (self):
 		self.ui = None
 
 	def getMusicShareDirectories(self):
-		#self.ui.statusBar.clearMessage()
 		self.ui.treeWidget.clear()
 		shares=self.xbmc.getMusicSources()
 		if shares.MType == 'S':
@@ -271,40 +278,24 @@ class XPotMainWindow (QMainWindow):
 				if "directories" in msobject.MResult:
 					self.addNavigatorRows(msobject, "directories")
 				if "files" in msobject.MResult:
-					self.addNavigatorRows(msobject.MResult, "files")
+					self.addNavigatorRows(msobject, "files")
 		else:
-			#self.ui.statusBar.showMessage(shares.MText)
 			print shares.MText	
 
 	def addNavigatorRows(self, msobject, type):
-		#self.ui.treeWidget.setFlags(Qt.ItemIsUserCheckable)
 		for item in msobject.MResult[type]:
 			item1 = QTreeWidgetItem()
 			item1.setText(0, item["label"])
 			item1.setText(1, item["file"])
 			item1.setText(2, type)
-			#item1.setForeground(0,brush)			
 			
 			if type == "files":
-				#item1.setFlags(Qt.ItemIsUserCheckable|Qt.ItemIsEnabled|Qt.ItemIsSelectable)
 				item1.setFlags(Qt.ItemIsEnabled|Qt.ItemIsSelectable)
 				item1.setCheckState(0,Qt.Unchecked);
 			else:
 				item1.setFlags(Qt.ItemIsUserCheckable|Qt.ItemIsEnabled|Qt.ItemIsSelectable)
 				item1.setCheckState(0,Qt.Unchecked);
-				
-			##color = QColor()
-			#color.setNamedColor("#787878")
-			#color.setAlpha(50)
-			##brush = QBrush()
-			#brush.setColor(color)
-			##itImg = QImage(":/img//metallic-holes-texture.jpg")
-			##brush.setTextureImage(itImg)
-			##item1.setBackground(0,brush)
-			#item1.setBackgroundColor(0,color)
-	
 			self.ui.treeWidget.addTopLevelItem(item1)
-			#self.ui.treeWidget.setProperty("FingerScrollable", True)
 
 	def getDirectory(self, dir):
 		msobject = self.xbmc.getMusicSourceObjects(dir)
@@ -317,16 +308,17 @@ class XPotMainWindow (QMainWindow):
 				self.addNavigatorRows(msobject, "files")
 				self.ui.treeWidget.setSelectionMode(QAbstractItemView.SingleSelection) 
 			self.ui.treeWidget.sortItems(0,Qt.AscendingOrder)
-			#self.ui.treeWidget.setProperty("FingerScrollable", True)		
 
 	
 	def setPlayButtonState(self):
+		exception = False
 		try:
 			playing = self.xbmc.getAudioPlayerState().MResult["playing"]
 			paused = self.xbmc.getAudioPlayerState().MResult["paused"]
 		except:
 			playing = False
 			paused = False
+			exception = True
 		if playing == True and paused == False:
 			self.ui.pushButtonPlay.setIcon(QIcon(":/img//media-playback-pause.png"))
 			self.ui.pushButtonPlay.setProperty("Paused",False)
@@ -334,7 +326,8 @@ class XPotMainWindow (QMainWindow):
 		else:
 			self.ui.pushButtonPlay.setIcon(QIcon(":/img//media-playback-start.png"))
 			self.ui.pushButtonPlay.setProperty("Paused",True)
-			self.currentSongThread.playerStopped = True
+			if exception == False:
+				self.currentSongThread.playerStopped = True
 		if playing == False:
 			return True
 		else:
@@ -357,6 +350,14 @@ class XPotMainWindow (QMainWindow):
 				self.xbmc.volumeDown()
 			sleep(0.1)				
 
+	def showMessage(self, message):
+		if message.find("Extra data") == -1: #Ignore JSONDecodeError -- Extra Data message
+			nmessage = message + "\nVerify that XBMC is switched on, accepting connections and accessible over this network connection."
+			QMessageBox.warning(self,'Warning', nmessage, buttons=QMessageBox.Ok, defaultButton=QMessageBox.NoButton)
+		else:
+			print message
+
+
 ##################################################################################
 #
 #  Event handlers for Signals
@@ -384,7 +385,17 @@ class XPotMainWindow (QMainWindow):
 			return True
 		return False
 
+	def clickDirectory(self, item):
+		if item.text(2) == "directories":
+			self.directoryClicked = True
+		else:
+			self.directoryClicked = False
+			
+
 	def click(self):
+		if self.directoryClicked:
+			self.directoryClicked = False
+			return
 		try:
 			items = self.ui.treeWidget.selectedItems()
 			dir = items[0].text(1)
@@ -450,12 +461,18 @@ class XPotMainWindow (QMainWindow):
 
 		
 	def addSelectedSongs(self):
-		#self.ui.statusBar.clearMessage()
+		self.progDialogAddCancelled = False
+		self.showProgress("Scanning Music Folders...", 0)
+		self.showProgress("Scanning Music Folders...", 5)
+		sleep(0.5)
+		self.showProgress("Scanning Music Folders...", 10)
+		
+		self.trackBuffer = []
 		self.ui.treeWidgetPlaylist.setColumnCount(1)
 		itemsList = []
 		playlist_length = self.ui.treeWidgetPlaylist.topLevelItemCount()
 		ic = 0
-		while ic < self.ui.treeWidget.topLevelItemCount():
+		while ic < self.ui.treeWidget.topLevelItemCount() and self.progDialogAddCancelled == False:
 			item = self.ui.treeWidget.topLevelItem(ic)
 			if item.text(2) == "files" and item.checkState(0) == Qt.Checked:
 				item.setCheckState(0,Qt.Unchecked)
@@ -463,22 +480,106 @@ class XPotMainWindow (QMainWindow):
 				item1.setText(0, item.text(0))
 				item1.setText(1, item.text(1))
 				item1.setText(2, item.text(2))
-				res=self.xbmc.addSong(item.text(1))
-				if res.MType == 'S':
-					itemsList.append(item1)
-				else:
-					#self.ui.statusBar.showMessage(res.MText)
-					print res.MText
+
+				itemsList.append(item1)
+			if item.text(2) == "directories" and item.checkState(0) == Qt.Checked:
+				item.setCheckState(0,Qt.Unchecked)
+				self.addSongsInDirectory(item.text(1))
 			ic = ic + 1
+
+		for itm in itemsList:
+			self.trackBuffer.append(itm.text(1))
+
+		track_count = self.trackBuffer.__len__()
+		
+		if self.randomize == True and self.progDialogAddCancelled == False:
+			self.showProgress("Shuffling Tracks...", 40)
+			cap = self.randomizeCap
+			if track_count < self.randomizeCap:
+				cap = track_count
+			if track_count > 0:
+				self.trackBuffer = random.sample(self.trackBuffer,cap)
+						
+		if self.progDialogAddCancelled == False:
+			self.showProgress("Adding %s Tracks to Playlist..." % str(track_count), 0)
+		if track_count > 0:
+			scale = Decimal(100) / Decimal(track_count)
+		progress = 0
+		for track in self.trackBuffer:	
+			if self.progDialogAddCancelled == True:
+				break		                                                                                                                                                                         
+			item2 = QTreeWidgetItem()
+			item2.setText(1, track)
+			res=self.xbmc.addSong(track)
+			itemsList.append(item2)
+			progress = progress + scale
+			self.showProgress("Adding %s Tracks to Playlist..." % str(track_count), progress)
+		
+		if progress < 100:
+			self.progDialogAdd.setValue(100)
+
 		ix = playlist_length + 1
 		self.ui.treeWidgetPlaylist.insertTopLevelItems(ix, itemsList)
 		self.ui.treeWidget.clearSelection()
 		self.shuffle()
 		self.currentSongIx = -1
+		self.randomize = False
+		self.ui.labelPlaylist.setText("Tracks: %s" % str(self.ui.treeWidgetPlaylist.topLevelItemCount()))
 		
+	def showProgress(self, title, progress):
+		#print self.progDialogAddCancelled
+		if self.progDialogAddCancelled == False:
+			self.progDialogAdd.setWindowTitle(title)
+			self.progDialogAdd.setValue(progress)
+		
+	def cancelProgressDialog(self):
+		print "cancelling progress..."
+		self.progDialogAdd.setValue(100)
+		self.progDialogAddCancelled = True
+		
+		
+	def addRandomSongs(self):
+		ic = 0
+		while ic < self.ui.treeWidget.topLevelItemCount():
+			item = self.ui.treeWidget.topLevelItem(ic)
+			if item.checkState(0) == Qt.Checked:		
+				self.randomize = True
+				self.randomizeCap, ok = QInputDialog.getInteger(self, "Shuffle", "Maximum Tracks", value=self.randomizeCap, min=5, max=2147483647, step=5)
+				if ok:
+					self.addSelectedSongs()
+				else:
+					self.randomize = False
+				break
+			ic = ic + 1
+		
+	
+	def addSongsInDirectory(self, item):
+		msobject = self.xbmc.getMusicSourceObjects(item)
+		if msobject.MType == 'S':
+			if 'directories' in msobject.MResult:
+				for itm in msobject.MResult["directories"]:
+					if self.progDialogAddCancelled:
+						return
+					self.showProgress("Scanning Music Folders...", 0)
+					self.addSongsInDirectory(itm['file'])
+					self.progDialogAdd.setValue(99)
+			elif 'files' in msobject.MResult:
+				file_count = msobject.MResult["files"].__len__()
+				scale = 99
+				progress = 0
+				if file_count > 0:
+					scale = 99 / file_count
+				for itm in sorted(msobject.MResult["files"], key=operator.itemgetter('file')):
+					if self.progDialogAddCancelled:
+						return
+					self.showProgress("Retrieving Music Tracks...", progress)
+					self.trackBuffer.append(itm['file'])
+					progress = progress + scale
+		else:
+			self.showMessage(msobject.MText)
+					
 
 	def addAllSongs(self):
-		#self.ui.statusBar.clearMessage()
 		self.ui.treeWidgetPlaylist.setColumnCount(1)
 		itemsList = []
 		playlist_length = self.ui.treeWidgetPlaylist.topLevelItemCount()
@@ -495,7 +596,6 @@ class XPotMainWindow (QMainWindow):
 				if res.MType == 'S':
 					itemsList.append(item1)
 				else:
-					#self.ui.statusBar.showMessage(res.MText)
 					print res.MText
 			ic = ic + 1
 		ix = playlist_length + 1
@@ -503,62 +603,57 @@ class XPotMainWindow (QMainWindow):
 		self.ui.treeWidget.clearSelection()
 		self.shuffle()
 		self.currentSongIx = -1
+		self.ui.labelPlaylist.setText("Tracks: %s" % str(self.ui.treeWidgetPlaylist.topLevelItemCount()))
 
 	def clearPlaylist(self):
-		#self.ui.statusBar.clearMessage()
 		res = self.xbmc.clearPlaylist()
 		if res.MType != 'S':
-			#self.ui.statusBar.showMessage(res.MText)
 			print res.MText
+			self.showMessage(res.MText)
 		self.ui.treeWidgetPlaylist.clear()
 		self.getCurrentPlaylistItems()					
 					
 	def playSongs(self):
-		#self.ui.statusBar.clearMessage()
 		stopped = self.setPlayButtonState()
 		if stopped:
 			res = self.xbmc.play()
 		else:
 			res = self.xbmc.pause()
 		if res.MType != 'S':
-			#self.ui.statusBar.showMessage(res.MText)
 			print res.MText
+			self.showMessage(res.MText)
 		self.setPlayButtonState()
 			
 	def stopPlaying(self):
-		#self.ui.statusBar.clearMessage()
 		res = self.xbmc.stop()
 		if res.MType != 'S':
-			#self.ui.statusBar.showMessage(res.MText)
 			print res.MText
+			self.showMessage(res.MText)
 		self.setPlayButtonState()	
 					
 
 	def skipNext(self):
-		#self.ui.statusBar.clearMessage()
 		res = self.xbmc.skipNext()
 		if res.MType != 'S':
-			#self.ui.statusBar.showMessage(res.MText)
 			print res.MText
+			self.showMessage(res.MText)
 		self.setPlayButtonState()			
 
 	def skipPrevious(self):
-		#self.ui.statusBar.clearMessage()
 		res = self.xbmc.skipPrevious()
 		if res.MType != 'S':
-			#self.ui.statusBar.showMessage(res.MText)
 			print res.MText
+			self.showMessage(res.MText)
 		self.setPlayButtonState()
 			
 	def shuffle(self):
-		#self.ui.statusBar.clearMessage()
 		if self.ui.pushButtonShuffle.isChecked():
 			res = self.xbmc.shuffle()			
 		else:
 			res = self.xbmc.unShuffle()
 		if res.MType != 'S':
-			#self.ui.statusBar.showMessage(res.MText)
 			print res.MText
+			self.showMessage(res.MText)
 		self.ui.treeWidgetPlaylist.clear()
 		self.getCurrentPlaylistItems()	
 		
